# Predict

Use the `teklia-dan predict` command to apply a trained DAN model on an image.

## Description of parameters

| Parameter                   | Description                                                                                     | Type    | Default       |
| --------------------------- | ----------------------------------------------------------------------------------------------- | ------- | ------------- |
| `--image`                   | Path to the image to predict. Must not be provided with `--image-dir`.                          | `Path`  |               |
| `--image-dir`               | Path to the folder where the images to predict are stored. Must not be provided with `--image`. | `Path`  |               |
| `--image-extension`         | The extension of the images in the folder. Ignored if `--image-dir` is not provided.            | `str`   | .jpg          |
| `--model`                   | Path to the model to use for prediction                                                         | `Path`  |               |
| `--parameters`              | Path to the YAML parameters file.                                                               | `Path`  |               |
| `--charset`                 | Path to the charset file.                                                                       | `Path`  |               |
| `--output`                  | Path to the output folder. Results will be saved in this directory.                             | `Path`  |               |
| `--confidence-score`        | Whether to return confidence scores.                                                            | `bool`  | `False`       |
| `--confidence-score-levels` | Level to return confidence scores. Should be any combination of `["line", "word", "char"]`.     | `str`   |               |
| `--attention-map`           | Whether to plot attention maps.                                                                 | `bool`  | `False`       |
| `--attention-map-scale`     | Image scaling factor before creating the GIF.                                                   | `float` | `0.5`         |
| `--attention-map-level`     | Level to plot the attention maps. Should be in `["line", "word", "char"]`.                      | `str`   | `"line"`      |
| `--predict-objects`         | Whether to return polygons coordinates.                                                         | `bool`  | `False`       |
| `--word-separators`         | List of word separators.                                                                        | `list`  | `[" ", "\n"]` |
| `--line-separators`         | List of line separators.                                                                        | `list`  | `["\n"]`      |
| `--threshold-method`        | Method to use for attention mask thresholding. Should be in `["otsu", "simple"]`.               | `str`   | `"otsu"`      |
| `--threshold-value `        | Threshold to use for the "simple" thresholding method.                                          | `int`   | `0`           |
| `--batch-size `             | Size of the batches for prediction.                                                             | `int`   | `1`           |

## Examples

### Predict with confidence scores

To run a prediction with confidence scores, run this command:

```shell
teklia-dan predict \
    --image dan_humu_page/example.jpg \
    --model dan_humu_page/model.pt \
    --parameters dan_humu_page/parameters.yml \
    --charset dan_humu_page/charset.pkl \
    --output dan_humu_page/predict/ \
    --confidence-score
```

It will create the following JSON file named `dan_humu_page/predict/example.json`

```json
{
  "text": "Hansteensgt. 2 IV 28/4 - 19\nKj\u00e6re Gerhard.\nTak for Brevet om Boken og Haven\nog Crokus og Blaaveis og tak fordi\nDu vilde be mig derut sammen\nmed Kris og Ragna. Men vet Du\nda ikke, at Kris reiste med sin S\u00f8-\nster Fru Cr\u00f8ger til Lillehammer\nnogle Dage efter Begravelsen? Hen\ndes Address er Amtsingeni\u00f8r\nCr\u00f8ger. Hun skriver at de blir\nder til lidt ut i Mai. Nu er hun\nnoksaa medtat skj\u00f8nner jeg af Sorg\nog af L\u00e6ngsel, skriver saameget r\u00f8-\nrende om Oluf. Ragna har det\nherligt, skriver hun. Hun er bare\ngla, og det vet jeg, at \"Oluf er gla over,\nder hvor han nu er. Jeg har saa in-\nderlig ondt af hende, og om Du skrev\net Par Ord tror jeg det vilde gj\u00f8re\nhende godt. - Jeg gl\u00e6der mig over,\nat Du har skrevet en Bok, og\njeg er vis paa, at den er god.",
  "confidence": 0.99
}
```

### Predict with confidence scores and line-level attention maps

To run a prediction with confidence scores and plot line-level attention maps, run this command:

```shell
teklia-dan predict \
    --image dan_humu_page/example.jpg \
    --model dan_humu_page/model.pt \
    --parameters dan_humu_page/parameters.yml \
    --charset dan_humu_page/charset.pkl \
    --output dan_humu_page/predict/ \
    --confidence-score \
    --attention-map \
```

It will create the following JSON file named `dan_humu_page/predict/example.json` and a GIF showing a word-level attention map `dan_humu_page/predict/example_line.gif`

```json
{
  "text": "Hansteensgt. 2 IV 28/4 - 19\nKj\u00e6re Gerhard.\nTak for Brevet om Boken og Haven\nog Crokus og Blaaveis og tak fordi\nDu vilde be mig derut sammen\nmed Kris og Ragna. Men vet Du\nda ikke, at Kris reiste med sin S\u00f8-\nster Fru Cr\u00f8ger til Lillehammer\nnogle Dage efter Begravelsen? Hen\ndes Address er Amtsingeni\u00f8r\nCr\u00f8ger. Hun skriver at de blir\nder til lidt ut i Mai. Nu er hun\nnoksaa medtat skj\u00f8nner jeg af Sorg\nog af L\u00e6ngsel, skriver saameget r\u00f8-\nrende om Oluf. Ragna har det\nherligt, skriver hun. Hun er bare\ngla, og det vet jeg, at \"Oluf er gla over,\nder hvor han nu er. Jeg har saa in-\nderlig ondt af hende, og om Du skrev\net Par Ord tror jeg det vilde gj\u00f8re\nhende godt. - Jeg gl\u00e6der mig over,\nat Du har skrevet en Bok, og\njeg er vis paa, at den er god.",
  "confidence": 0.99,
  "attention_gif": "dan_humu_page/predict/example_line.gif"
}
```

<img src="../../assets/example_line.gif" />

### Predict with confidence scores and word-level attention maps

To run a prediction with confidence scores and plot word-level attention maps, run this command:

```shell
teklia-dan predict \
    --image dan_humu_page/example.jpg \
    --model dan_humu_page/model.pt \
    --parameters dan_humu_page/parameters.yml \
    --charset dan_humu_page/charset.pkl \
    --output dan_humu_page/predict/ \
    --confidence-score \
    --attention-map \
    --attention-map-level word \
    --attention-map-scale 0.5
```

It will create the following JSON file named `dan_humu_page/predict/example.json` and a GIF showing a word-level attention map `dan_humu_page/predict/example_word.gif`.

```json
{
  "text": "Hansteensgt. 2 IV 28/4 - 19\nKj\u00e6re Gerhard.\nTak for Brevet om Boken og Haven\nog Crokus og Blaaveis og tak fordi\nDu vilde be mig derut sammen\nmed Kris og Ragna. Men vet Du\nda ikke, at Kris reiste med sin S\u00f8-\nster Fru Cr\u00f8ger til Lillehammer\nnogle Dage efter Begravelsen? Hen\ndes Address er Amtsingeni\u00f8r\nCr\u00f8ger. Hun skriver at de blir\nder til lidt ut i Mai. Nu er hun\nnoksaa medtat skj\u00f8nner jeg af Sorg\nog af L\u00e6ngsel, skriver saameget r\u00f8-\nrende om Oluf. Ragna har det\nherligt, skriver hun. Hun er bare\ngla, og det vet jeg, at \"Oluf er gla over,\nder hvor han nu er. Jeg har saa in-\nderlig ondt af hende, og om Du skrev\net Par Ord tror jeg det vilde gj\u00f8re\nhende godt. - Jeg gl\u00e6der mig over,\nat Du har skrevet en Bok, og\njeg er vis paa, at den er god.",
  "confidence": 0.99,
  "attention_gif": "dan_humu_page/predict/example_word.gif"
}
```

<img src="../../assets/example_word.gif" >

### Predict with line-level attention maps and extract polygons

To run a prediction, plot line-level attention maps, and extract polygons, run this command:

```shell
teklia-dan predict \
    --image dan_humu_page/example.jpg \
    --model dan_humu_page/model.pt \
    --parameters dan_humu_page/parameters.yml \
    --charset dan_humu_page/charset.pkl \
    --output dan_humu_page/predict/ \
    --attention-map \
    --predict-objects \
    --threshold-method otsu
```

It will create the following JSON file named `dan_humu_page/predict/example.json` and a GIF showing a line-level attention map with extracted polygons `dan_humu_page/predict/example_line.gif`

```json
{
  "text": "Oslo\n39 \nOresden den 24te Rasser!\nH\u00f8jst\u00e6redesherr Hartvig - assert!\nUllereder fra den f\u00f8rste tide da\njeg havder den tilfredsstillelser at vide den ar-\ndistiske ledelser af Kristiania theater i Deres\nhronder, har jeg g\u00e5t hernede med et stille\nh\u00e5b om fra Dem at modtage et forelag, sig -\nsende tils at lade \"K\u00e6rlighedens \u00abKomedie\u00bb\nopf\u00f8re fore det norske purblikum.\nEt s\u00e5dant forslag er imidlertid, imod\nforventning; ikke fremkommet, og jeg n\u00f8des der-\nfor tils self at grivbe initiativet, hvilket hervede\nsker, idet jeg\nbeder\nbet\nragte stigkket some ved denne\nskrivelse officielde indleveret til theatret. No-\nget exemplar af bogen vedlagger jeg ikke da\ndenne (i 2den udgave) med Lethed kan er -\nholdet deroppe.\nDe bet\u00e6nkeligheder, jeg i sin tid n\u00e6-\nrede mod stykkets opf\u00f8relse, er for l\u00e6nge si -\ndem forsvundne. Af mange begn er jeg kom-\nmen til den overbevisning at almenlreden\naru har f\u00e5tt sine \u00f8gne opladte for den sand -\nMed at dette arbejde i sin indersten id\u00e9 hviler\np\u00e5 et ubedinget meralsk grundlag, og brad\nstykkets hele kunstneriske struktuve ang\u00e5r,",
  "objects": [
    {
      "confidence": 0.68,
      "polygon": [
        [
          264,
          118
        ],
        [
          410,
          118
        ],
        [
          410,
          185
        ],
        [
          264,
          185
        ]
      ],
      "text": "Oslo",
      "text_confidence": 0.8
    }
  ],
  "attention_gif": "dan_humu_page/predict/example_line.gif"
}
```

<img src="../../assets/example_line_polygon.gif" >
